import inspect
from uuid import uuid4
from functools import partial
from typing import Any, Union, Optional, cast

from pydantic import parse_obj_as
from nonebot.typing import overrides
from nonebot.message import handle_event

from nonebot.adapters import Adapter
from nonebot.adapters import Bot as BaseBot

from .api import API
from .config import BotConfig
from .exception import ApiNotAvailable
from .model import InputMedia, MessageEntity
from .message import File, Entity, Message, UnCombinFile, MessageSegment
from .event import (
    Event,
    MessageEvent,
    EventWithChat,
    GroupMessageEvent,
    PrivateMessageEvent,
)


class Bot(BaseBot, API):
    """
    Telegram Bot 适配。继承属性参考 `BaseBot <./#class-basebot>`_ 。
    """

    def __init__(self, adapter: "Adapter", config: BotConfig):
        self.adapter = adapter
        self.self_id = config.token.split(":")[0]
        self.username: Optional[str] = None
        self.bot_config = config
        self.secret_token = uuid4().hex

    def _check_tome(self, event: MessageEvent):
        def process_first_segment(message: Message):
            if not message:
                message.append(Entity.text(""))
            elif message[0].is_text():
                message[0].data["text"] = message[0].data["text"].lstrip()
                if not str(message[0]):
                    del message[0]
                    process_first_segment(message)

        if (
            event.reply_to_message
            and (
                isinstance(event.reply_to_message, GroupMessageEvent)
                or isinstance(event.reply_to_message, PrivateMessageEvent)
            )
            and str(event.reply_to_message.from_.id) == self.self_id
        ):
            event._tome = True
            return

        segment = event.message[0]
        if segment.type == "mention":
            if segment.data.get("text", "")[1:] == self.username:
                del event.message[0]
                process_first_segment(event.message)
                event._tome = True
        elif segment.type == "text":
            text = str(segment)
            for nickname in self.config.nickname:
                if nickname in text:
                    event.message[0].data["text"] = text.replace(nickname, "", 1)
                    process_first_segment(event.message)
                    event._tome = True
                    break

    async def handle_event(self, event: Event):
        if isinstance(event, MessageEvent):
            self._check_tome(event)
        await handle_event(self, event)

    async def call_api(self, api: str, *args: Any, **kargs: Any) -> Any:
        if hasattr(API, api):
            sign = inspect.signature(getattr(API, api))
            args_ = list(args)
            for param in sign.parameters.values():
                if param.name != "self" and param.name not in kargs:
                    try:
                        kargs[param.name] = args_.pop(0)
                    except IndexError:
                        kargs[param.name] = None
            return parse_obj_as(
                sign.return_annotation, await super().call_api(api, **kargs)
            )
        else:
            return await super().call_api(api, **kargs)

    def __getattribute__(self, __name: str) -> Any:
        if not __name.startswith("__") and hasattr(API, __name):
            return partial(self.call_api, __name)
        else:
            return object.__getattribute__(self, __name)

    # TODO 重构
    @overrides(BaseBot)
    async def send(
        self,
        event: Event,
        message: Union[str, Message, MessageSegment],
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
        reply_to_message_id: Optional[int] = None,
        allow_sending_without_reply: Optional[bool] = None,
        **kwargs,
    ) -> Any:
        """
        由于 Telegram 对于不同类型的消息有不同的 API，如果需要使用同一方法发送不同类型的消息请使用此方法。

        - `File` 或 `Entity` 可随意组合
        - 非 `File` 非 `Entity` 的 `MessageSegment` 无法组合
        """
        kwargs.update(
            {
                "disable_notification": disable_notification,
                "protect_content": protect_content,
                "reply_to_message_id": reply_to_message_id,
                "allow_sending_without_reply": allow_sending_without_reply,
            }
        )
        if isinstance(event, EventWithChat):
            message_thread_id = cast(
                Optional[int], getattr(event, "message_thread_id", None)
            )
            if isinstance(message, str):
                return await self.send_message(
                    chat_id=event.chat.id,
                    message_thread_id=message_thread_id,
                    text=message,
                    **kwargs,
                )
            elif isinstance(message, MessageSegment):
                if message.is_text():
                    return await self.send_message(
                        chat_id=event.chat.id,
                        message_thread_id=message_thread_id,
                        text=str(message),
                        entities=[
                            MessageEntity(
                                type=message.type,
                                offset=0,
                                length=len(message.data["text"]),
                                url=message.data.get("url"),
                                user=message.data.get("user"),
                                language=message.data.get("language"),
                            )
                        ]
                        if message.type != "text"
                        else None,
                        **kwargs,
                    )
                elif isinstance(message, File):
                    return await self.call_api(
                        f"send_{message.type}",
                        chat_id=event.chat.id,
                        message_thread_id=message_thread_id,
                        **{message.type: message.data["file"]},
                        **kwargs,
                    )
                else:
                    return await self.call_api(
                        f"send_{message.type}",
                        chat_id=event.chat.id,
                        message_thread_id=message_thread_id,
                        **message.data,
                        **kwargs,
                    )
            else:
                entities = Message(filter(lambda x: isinstance(x, Entity), message))
                files = Message(
                    filter(
                        lambda x: isinstance(x, File)
                        and not isinstance(message, UnCombinFile),
                        message,
                    )
                )
                others = Message(
                    filter(
                        lambda x: not (isinstance(x, Entity) or isinstance(x, File))
                        or isinstance(message, UnCombinFile),
                        message,
                    )
                )
                if others:
                    if len(others) > 1 or files or entities:
                        raise ApiNotAvailable
                elif files:
                    if len(files) > 1:
                        return await self.send_media_group(
                            chat_id=event.chat.id,
                            message_thread_id=message_thread_id,
                            media=[
                                InputMedia(
                                    type=files[0].type,
                                    media=files[0].data["file"],
                                    caption=str(entities),
                                    caption_entities=[
                                        MessageEntity(
                                            type=entity.type,
                                            offset=sum(map(len, message[:i])),
                                            length=len(entity.data["text"]),
                                            url=entity.data.get("url"),
                                            user=entity.data.get("user"),
                                            language=entity.data.get("language"),
                                        )
                                        for i, entity in enumerate(message)
                                        if entity.is_text() and entity.type != "text"
                                    ],
                                )
                            ]
                            + [
                                InputMedia(
                                    type=file.type,
                                    media=file.data["file"],
                                )
                                for file in files[1:]
                            ],  # type:ignore
                            **kwargs,
                        )

                    else:
                        file = files[0]
                        return await self.call_api(
                            f"send_{file.type}",
                            chat_id=event.chat.id,
                            message_thread_id=message_thread_id,
                            **{
                                file.type: file.data.get("file"),
                                "caption": str(entities) if entities else None,
                                "caption_entities": [
                                    MessageEntity(
                                        type=entity.type,
                                        offset=sum(map(len, message[:i])),
                                        length=len(entity.data["text"]),
                                        url=entity.data.get("url"),
                                        user=entity.data.get("user"),
                                        language=entity.data.get("language"),
                                    )
                                    for i, entity in enumerate(message)
                                    if entity.is_text() and entity.type != "text"
                                ]
                                if entities
                                else None,
                            },
                            **kwargs,
                        )
                else:
                    return await self.send_message(
                        chat_id=event.chat.id,
                        message_thread_id=message_thread_id,
                        text=str(message),
                        entities=[
                            MessageEntity(
                                type=entity.type,
                                offset=sum(map(len, message[:i])),
                                length=len(entity.data["text"]),
                                url=entity.data.get("url"),
                                user=entity.data.get("user"),
                                language=entity.data.get("language"),
                            )
                            for i, entity in enumerate(message)
                            if entity.is_text() and entity.type != "text"
                        ],
                        **kwargs,
                    )
        else:
            raise ApiNotAvailable



import json
from pathlib import Path

import pytest


@pytest.mark.asyncio
async def test_event():
    from nonebot.adapters.telegram import Event
    from nonebot.adapters.telegram.event import (
        InlineQueryEvent,
        CallbackQueryEvent,
        PrivateMessageEvent,
        ChosenInlineResultEvent,
        PrivateEditedMessageEvent,
    )

    with (Path(__file__).parent / "updates.json").open("r", encoding="utf8") as f:
        test_updates = json.load(f)

    update_data = test_updates[0]
    event = Event.parse_event(update_data)
    assert isinstance(event, PrivateMessageEvent)

    update_data = test_updates[1]
    event = Event.parse_event(update_data)
    assert isinstance(event, PrivateMessageEvent)

    update_data = test_updates[2]
    event = Event.parse_event(update_data)
    assert isinstance(event, PrivateMessageEvent)

    update_data = test_updates[3]
    event = Event.parse_event(update_data)
    assert isinstance(event, PrivateMessageEvent)

    update_data = test_updates[4]
    event = Event.parse_event(update_data)
    assert isinstance(event, PrivateEditedMessageEvent)

    update_data = test_updates[9]
    event = Event.parse_event(update_data)
    assert isinstance(event, InlineQueryEvent)

    update_data = test_updates[10]
    event = Event.parse_event(update_data)
    assert isinstance(event, ChosenInlineResultEvent)

    update_data = test_updates[11]
    event = Event.parse_event(update_data)
    assert isinstance(event, CallbackQueryEvent)



from typing import Any, Dict, List, Type, Union, Literal, Iterable, Optional

from nonebot.typing import overrides

from nonebot.adapters import Message as BaseMessage
from nonebot.adapters import MessageSegment as BaseMessageSegment

from .model import User, LabeledPrice


class MessageSegment(BaseMessageSegment):
    """
    telegram 协议 MessageSegment 适配。具体方法参考协议消息段类型或源码。
    """

    @classmethod
    @overrides(BaseMessageSegment)
    def get_message_class(cls) -> Type["Message"]:
        return Message

    @overrides(BaseMessageSegment)
    def __str__(self) -> str:
        if self.is_text():
            return self.data.get("text", "")
        return ""

    def __repr__(self) -> str:
        if self.is_text():
            return self.data.get("text", "")
        params = ", ".join([f"{k}={v}" for k, v in self.data.items() if v is not None])
        return f"[{self.type}{':' if params else ''}{params}]"

    @overrides(BaseMessageSegment)
    def is_text(self) -> bool:
        return False

    # TODO need test
    @staticmethod
    def location(latitude: float, longitude: float) -> "MessageSegment":
        return MessageSegment(
            "location", {"latitude": latitude, "longitute": longitude}
        )

    # TODO need test
    @staticmethod
    def venue(
        latitude: float, longitude: float, title: str, address: str
    ) -> "MessageSegment":
        return MessageSegment("venue", {"latitude": latitude, "longitute": longitude})

    # TODO DELAY
    @staticmethod
    def contact():
        pass

    @staticmethod
    def poll(question: str, options: List[str]) -> "MessageSegment":
        return MessageSegment("poll", {"question": question, "options": options})

    @staticmethod
    def dice(emoji: Literal["🎲", "🎯", "🏀", "⚽", "🎳", "🎰"] = "🎲") -> "MessageSegment":
        return MessageSegment("dice", {"question": emoji})

    @staticmethod
    def chat_action(
        action: Literal[
            "typing",
            "upload_photo",
            "record_video",
            "upload_video",
            "record_voice",
            "upload_voice",
            "upload_document",
            "choose_sticker",
            "find_location",
            "record_video_note",
            "upload_video_note",
        ]
    ) -> "MessageSegment":
        """
        仅发送，用于提醒用户机器人正在准备什么
        """
        return MessageSegment("chat_action", {"action": action})

    # TODO DELAY
    @staticmethod
    def invoice(
        title: str,
        description: str,
        payload: str,
        provider_token: str,
        currency: str,
        prices: List[LabeledPrice],
    ):
        raise NotImplementedError

    # TODO DELAY
    @staticmethod
    def game(game_short_name: str):
        raise NotImplementedError


class Entity(MessageSegment):
    @overrides(BaseMessageSegment)
    def is_text(self) -> bool:
        return True

    @staticmethod
    def text(text: str):
        """
        纯文本
        """
        return Entity("text", {"text": text})

    @staticmethod
    def mention(text: str):
        return Entity("mention", {"text": text})

    @staticmethod
    def hashtag(text: str):
        return Entity("hashtag", {"text": text})

    @staticmethod
    def cashtag(text: str):
        return Entity("cashtag", {"text": text})

    @staticmethod
    def bot_command(text: str):
        return Entity("bot_command", {"text": text})

    @staticmethod
    def url(text: str):
        return Entity("url", {"text": text})

    @staticmethod
    def email(text: str):
        return Entity("email", {"text": text})

    @staticmethod
    def phone_number(text: str):
        return Entity("phone_number", {"text": text})

    @staticmethod
    def bold(text: str):
        return Entity("bold", {"text": text})

    @staticmethod
    def italic(text: str):
        return Entity("italic", {"text": text})

    @staticmethod
    def underline(text: str):
        return Entity("underline", {"text": text})

    @staticmethod
    def strikethrough(text: str):
        return Entity("strikethrough", {"text": text})

    @staticmethod
    def spoiler(text: str):
        return Entity("spoiler", {"text": text})

    @staticmethod
    def code(text: str):
        return Entity("code", {"text": text})

    @staticmethod
    def pre(text: str, language: str) -> "Entity":
        return Entity("pre", {"text": text, "language": language})

    @staticmethod
    def text_link(text: str, url: str) -> "Entity":
        return Entity("text_link", {"text": text, "url": url})

    @staticmethod
    def text_mention(text: str, user: User) -> "Entity":
        return Entity("text_mention", {"text": text, "user": user})

    @staticmethod
    def custom_emoji(text: str, custom_emoji_id: str) -> "Entity":
        return Entity(
            "custom_emoji", {"text": text, "custom_emoji_id": custom_emoji_id}
        )


class File(MessageSegment):
    @staticmethod
    def photo(
        file: Union[str, bytes], has_spoiler: Optional[bool] = None
    ) -> "MessageSegment":
        return File("photo", {"file": file, "has_spoiler": has_spoiler})

    @staticmethod
    def voice(
        file: Union[str, bytes],
    ) -> "MessageSegment":
        return File("voice", {"file": file})

    @staticmethod
    def animation(
        file: Union[str, bytes],
        thumbnail: Union[None, str, bytes] = None,
        has_spoiler: Optional[bool] = None,
    ) -> "MessageSegment":
        return File(
            "animation",
            {"file": file, "thumbnail": thumbnail, "has_spoiler": has_spoiler},
        )

    @staticmethod
    def audio(
        file: Union[str, bytes],
        thumbnail: Union[None, str, bytes] = None,
    ) -> "MessageSegment":
        return File("audio", {"file": file, "thumbnail": thumbnail})

    @staticmethod
    def document(
        file: Union[str, bytes],
        thumbnail: Union[None, str, bytes] = None,
    ) -> "MessageSegment":
        return File("document", {"file": file, "thumbnail": thumbnail})

    @staticmethod
    def video(
        file: Union[str, bytes],
        thumbnail: Union[None, str, bytes] = None,
        has_spoiler: Optional[bool] = None,
    ) -> "MessageSegment":
        return File(
            "video", {"file": file, "thumbnail": thumbnail, "has_spoiler": has_spoiler}
        )


class UnCombinFile(File):
    @staticmethod
    def sticker(file: Union[str, bytes]) -> "MessageSegment":
        return File("sticker", {"file": file})

    @staticmethod
    def video_note(
        file: Union[str, bytes],
        thumbnail: Union[None, str, bytes] = None,
    ) -> "MessageSegment":
        return File("video_note", {"file": file, "thumbnail": thumbnail})


class Message(BaseMessage[MessageSegment]):
    def __repr__(self) -> str:
        return "".join(repr(seg) for seg in self)

    @classmethod
    @overrides(BaseMessage)
    def get_segment_class(cls) -> Type["MessageSegment"]:
        return MessageSegment

    @staticmethod
    @overrides(BaseMessage)
    def _construct(msg: str) -> Iterable[MessageSegment]:
        yield Entity.text(msg)

    @classmethod
    def parse_obj(cls, obj: Dict[str, Any]) -> "Message":
        msg = []
        if "text" in obj or "caption" in obj:
            key, entities_key = (
                ("text", "entities")
                if "text" in obj
                else ("caption", "caption_entities")
            )
            offset = 0
            for entity in obj.get(entities_key, ()):
                if entity["offset"] > offset:
                    msg.append(
                        Entity("text", {"text": obj[key][offset : entity["offset"]]})
                    )
                nb_entity = Entity(
                    entity["type"],
                    {
                        "text": obj[key][
                            entity["offset"] : entity["offset"] + entity["length"]
                        ]
                    },
                )
                if "language" in entity:
                    nb_entity.data["language"] = entity["language"]
                if "url" in entity:
                    nb_entity.data["url"] = entity["url"]
                if "user" in entity:
                    nb_entity.data["user"] = User(**entity["user"])
                msg.append(nb_entity)
                offset = entity["offset"] + entity["length"]
            if offset < len(obj[key]):
                msg.append(Entity("text", {"text": obj[key][offset:]}))
            del obj[key]
            obj.pop(entities_key, None)
        kargs = {}
        if obj.get("has_media_spoiler", None):
            kargs["has_spoiler"] = True
        obj.pop("has_media_spoiler", None)
        if "animation" in obj:
            del obj["document"]
        for key in tuple(obj.keys()):
            if key == "photo":
                seg = File("photo", {"file": obj[key][-1]["file_id"], **kargs})
            elif key in ("voice", "audio", "animation", "document", "video"):
                seg = File(key, {"file": obj[key]["file_id"]})
            elif key in ("sticker", "video_note"):
                seg = UnCombinFile(key, {"file": obj[key]["file_id"]})
            elif key == "dice":
                seg = MessageSegment(
                    key, {"emoji": obj[key]["emoji"], "value": obj[key]["value"]}
                )
            elif key == "poll":
                seg = MessageSegment(
                    key,
                    {"question": obj[key]["question"], "options": obj[key]["options"]},
                )
            else:
                continue
            if isinstance(obj[key], dict) and obj[key].get("thumbnail", None):
                kargs["thumbnail"] = obj[key]["thumbnail"]["file_id"]
            seg.data.update(kargs)
            msg.append(seg)
            del obj[key]
        return cls(msg)



import json
import asyncio
import pathlib
from typing import Any, Dict, List, cast

from anyio import open_file
from pydantic.main import BaseModel
from nonebot.typing import overrides
from pydantic.json import pydantic_encoder
from nonebot.utils import escape_tag, logger_wrapper
from nonebot.drivers import URL, Driver, Request, Response, HTTPServerSetup

from nonebot.adapters import Adapter as BaseAdapter

from .bot import Bot
from .event import Event
from .model import InputMedia
from .config import BotConfig, AdapterConfig
from .exception import (
    ActionFailed,
    NetworkError,
    ApiNotAvailable,
    TelegramAdapterException,
)


def _escape_none(data: Dict[str, Any]) -> Dict[str, Any]:
    return {k: v for k, v in data.items() if v is not None}


log = logger_wrapper("Telegram")


class Adapter(BaseAdapter):
    @overrides(BaseAdapter)
    def __init__(self, driver: Driver, **kwargs: Any):
        super().__init__(driver, **kwargs)
        self.adapter_config = AdapterConfig(**self.config.dict())
        self.tasks: List[asyncio.Task] = []
        self.setup()

    @classmethod
    @overrides(BaseAdapter)
    def get_name(cls) -> str:
        return "Telegram"

    def setup_webhook(self, bot: Bot):
        @self.driver.on_startup
        async def _():
            try:
                bot.username = (await bot.get_me()).username
                log("INFO", "Delete old webhook")
                await bot.delete_webhook()
                log("INFO", "Set new webhook")
                await bot.set_webhook(
                    url=f"{self.adapter_config.telegram_webhook_url}/telegram",
                    secret_token=bot.secret_token,
                )
                self.bot_connect(bot)
            except Exception as e:
                log("ERROR", f"Setup for bot {bot.self_id} failed", e)

    async def poll(self, bot: Bot):
        try:
            bot.username = (await bot.get_me()).username
            log("INFO", "Delete old webhook")
            await bot.delete_webhook()
            log("INFO", "Start poll")
            self.bot_connect(bot)

            update_offset = None
            while True:
                try:
                    updates = await bot.get_updates(offset=update_offset, timeout=30)
                    if update_offset is not None:
                        for update in updates:
                            update_offset = update.update_id + 1
                            event = Event.parse_event(
                                update.dict(by_alias=True, exclude_none=True)
                            )
                            log(
                                "DEBUG",
                                escape_tag(
                                    str(
                                        event.dict(
                                            exclude_none=True,
                                            exclude={"telegram_model"},
                                        )
                                    )
                                ),
                            )
                            await bot.handle_event(event)
                    elif updates:
                        update_offset = updates[0].update_id
                except Exception as e:
                    log("ERROR", f"Get updates for bot {bot.self_id} failed", e)
        except Exception as e:
            log("ERROR", f"Setup for bot {bot.self_id} failed", e)

    def setup_polling(self, bot: Bot):
        @self.driver.on_startup
        async def _():
            self.tasks.append(asyncio.create_task(self.poll(bot)))

        @self.driver.on_shutdown
        async def _():
            for task in self.tasks:
                if not task.done():
                    task.cancel()
            await asyncio.gather(*self.tasks, return_exceptions=True)

    async def handle_http(self, request: Request) -> Response:
        token = request.headers.get("X-Telegram-Bot-Api-Secret-Token")
        for bot in self.bots.values():
            bot = cast(Bot, bot)
            if bot.secret_token == token:
                if request.content:
                    update: dict = json.loads(request.content)
                    asyncio.create_task(bot.handle_event(Event.parse_event(update)))
                return Response(204)
        return Response(401)

    def setup(self) -> None:
        self.setup_http_server(
            HTTPServerSetup(
                URL("/telegram"),
                "POST",
                self.get_name(),
                self.handle_http,
            )
        )
        for bot_config in self.adapter_config.telegram_bots:
            bot = Bot(self, bot_config)
            if bot_config.is_webhook:
                self.setup_webhook(bot)
            else:
                self.setup_polling(bot)

    @overrides(BaseAdapter)
    async def _call_api(self, bot: Bot, api: str, **data) -> Any:
        # 将方法名称改为驼峰式
        api = api.split("_", maxsplit=1)[0] + "".join(
            s.capitalize() for s in api.split("_")[1:]
        )
        data = _escape_none(data)

        # 分离文件到 files
        files = {}
        if api == "sendMediaGroup":
            upload_count = 0
            for media in data["media"]:
                media = cast(InputMedia, media)
                if isinstance(media.media, bytes):
                    files[f"upload{upload_count}"] = (
                        f"upload{upload_count}",
                        media.media,
                    )
                    media.media = f"attach://upload{upload_count}"
                elif (file := pathlib.Path(media.media)).is_file():
                    async with await open_file(media.media, "rb") as f:
                        files[file.name] = (file.name, await f.read())
                    media.media = f"attach://{pathlib.Path(media.media).name}"
        elif api in (
            "sendPhoto",
            "sendAudio",
            "sendDocument",
            "sendVideo",
            "sendAnimation",
            "sendVoice",
            "sendVideoNote",
        ):
            type = api[4:].lower()
            for key in (type, "thumb"):
                if (value := data.pop(key, None)) is None:
                    continue
                if isinstance(value, bytes):
                    files[key] = ("upload", value)
                elif (
                    isinstance(value, str)
                    and (file := pathlib.Path(str(value))).is_file()
                ):
                    async with await open_file(value, "rb") as f:
                        files[key] = (file.name, await f.read())
                else:
                    data[key] = value

        # 最后处理 data 以符合 DataTypes
        for key in data:
            if not isinstance(data[key], str):
                data[key] = json.dumps(
                    data[key],
                    default=(
                        lambda o: o.dict(exclude_none=True)
                        if isinstance(o, BaseModel)
                        else pydantic_encoder(o)
                    ),
                )

        request = Request(
            "POST",
            f"{bot.bot_config.api_server}bot{bot.bot_config.token}/{api}",
            data=data if files else None,
            json=data if not files else None,
            files=files,
            proxy=self.adapter_config.proxy,
        )
        try:
            response = await self.request(request)
            if response.content:
                if 200 <= response.status_code < 300:
                    return json.loads(response.content)["result"]
                elif 400 <= response.status_code < 404:
                    raise ActionFailed(json.loads(response.content)["description"])
                elif response.status_code == 404:
                    raise ApiNotAvailable
                raise NetworkError(
                    f"HTTP request received unexpected {response.status_code} {response.content}"
                )
            else:
                raise ValueError("Empty response")
        except TelegramAdapterException:
            raise
        except Exception as e:
            raise NetworkError("HTTP request failed") from e



from nonebot.permission import Permission

from .bot import Bot
from .event import GroupMessageEvent, PrivateMessageEvent


async def _private(event: PrivateMessageEvent) -> bool:
    return True


PRIVATE: Permission = Permission(_private)
""" 匹配任意私聊消息类型事件"""


async def _group(event: GroupMessageEvent) -> bool:
    return True


async def _group_member(bot: Bot, event: GroupMessageEvent) -> bool:
    return (await bot.get_chat_member(event.chat.id, event.from_.id)).status == "member"


async def _group_admin(bot: Bot, event: GroupMessageEvent) -> bool:
    return (
        await bot.get_chat_member(event.chat.id, event.from_.id)
    ).status == "administrator"


async def _group_creator(bot: Bot, event: GroupMessageEvent) -> bool:
    return (
        await bot.get_chat_member(event.chat.id, event.from_.id)
    ).status == "creator"


GROUP: Permission = Permission(_group)
"""匹配任意群聊消息类型事件"""
GROUP_MEMBER: Permission = Permission(_group_member)
"""匹配任意群员群聊消息类型事件

:::warning 警告
该权限通过 event.sender 进行判断且不包含管理员以及群创建者！
:::
"""
GROUP_ADMIN: Permission = Permission(_group_admin)
"""匹配任意群管理员群聊消息类型事件"""
GROUP_CREATOR: Permission = Permission(_group_creator)
"""匹配任意群创建者群聊消息类型事件"""



from typing import Optional

from nonebot.exception import AdapterException
from nonebot.exception import ActionFailed as BaseActionFailed
from nonebot.exception import NetworkError as BaseNetworkError
from nonebot.exception import NoLogException as BaseNoLogException
from nonebot.exception import ApiNotAvailable as BaseApiNotAvailable


class TelegramAdapterException(AdapterException):
    def __init__(self):
        super().__init__("Telegram")


class NoLogException(BaseNoLogException, TelegramAdapterException):
    pass


class ActionFailed(BaseActionFailed, TelegramAdapterException):
    """
    :说明:
      API 请求返回错误信息。
    """

    def __init__(self, description: Optional[str] = None):
        super().__init__()
        self.description = description

    def __repr__(self):
        return f"<ActionFailed {self.description}>"

    def __str__(self):
        return self.__repr__()


class NetworkError(BaseNetworkError, TelegramAdapterException):
    """
    :说明:
      网络错误。
    """

    def __init__(self, msg: Optional[str] = None):
        super().__init__()
        self.msg = msg

    def __repr__(self):
        return f"<NetWorkError message={self.msg}>"

    def __str__(self):
        return self.__repr__()


class ApiNotAvailable(BaseApiNotAvailable, TelegramAdapterException):
    pass
